import bcrypt from 'bcryptjs'
import UserRepository from '../repositories/userRepository'
import prisma from '../utilities/prisma'
import { User } from '@prisma/client'
import config from '../utilities/config'

/**
 * User service
 * Handles user login and registration
 * @class UserService
 */
class UserService {
  /**
   * Login user
   * @param username
   * @param password
   */
  async login(username: string, password: string): Promise<boolean | User> {
    const user = await UserRepository.getByUsername(username)
    if (!user) {
      return false
    }

    const passwordMatch = await bcrypt.compare(password, user.password)
    if (!passwordMatch) {
      return false
    }

    return user
  }

  /**
   * Register user
   * @param username
   * @param password
   */
  async register(username: string, email: string, password: string): Promise<boolean | User> {
    const user = await UserRepository.getByUsername(username)
    if (user) {
      return false
    }

    const userByEmail = await UserRepository.getByEmail(email)
    if (userByEmail) {
      return false
    }

    const hashedPassword = await bcrypt.hash(password, 10)
    return prisma.user.create({
      data: {
        username,
        email,
        password: hashedPassword
      }
    })
  }

  /**
   * Reset password
   * @param email
   */
  async resetPassword(email: string): Promise<boolean | User> {
    const nodemailer = require("nodemailer");

    const transporter = nodemailer.createTransport({
      host: config.SMTP_HOST,
      port: config.SMTP_PORT,
      secure: false,
      auth: {
        user: config.SMTP_USER,
        pass: config.SMTP_PASSWORD,
      },
    });

    const info = await transporter.sendMail({
      from: config.SMTP_USER,
      to: email,
      subject: "Reset your password",
      text: "A password reset has been requested, reset your password here: ", // Plain text body
      html: "<p>A password reset has been requested, reset your password here: </p>", // Html body
    });
    console.log("Message sent: %s", info.messageId);

    return info.messageId
  }
}

export default UserService
